@@ -87,6 +87,14 @@ pub(crate) fn register(
         drop_tcp_listener,
         namespace_filter,
     )?;
+    link_if_match(
+        linker,
+        "lunatic::networking",
+        "local_addr",
+        FuncType::new([ValType::I64, ValType::I32], [ValType::I32]),
+        local_addr,
+        namespace_filter,
+    )?;
     link_async3_if_match(
         linker,
         "lunatic::networking",
@@ -185,6 +193,7 @@ pub(crate) fn register(
 //% Returns:
 //% * 0 on success - The ID of the newly created DNS iterator is written to **id_u64_ptr**
 //% * 1 on error   - The error ID is written to **id_u64_ptr**
+//% * 9027 if the operation timed out
 //%
 //% Performs a DNS resolution. The returned iterator may not actually yield any values
 //% depending on the outcome of any resolution performed.
@@ -238,12 +247,7 @@ fn resolve(
             Ok(result)
         } else {
             // Call timed out
-            let error = std::io::Error::new(std::io::ErrorKind::TimedOut, "Resolve call timed out");
-            let error_id = caller.data_mut().errors.add(error.into());
-            memory
-                .write(&mut caller, id_u64_ptr as usize, &error_id.to_le_bytes())
-                .or_trap("lunatic::networking::resolve")?;
-            Ok(1)
+            Ok(9027)
         };
         return_
     })
@@ -417,7 +421,7 @@ fn tcp_bind(
 //% Drops the TCP listener resource.
 //%
 //% Traps:
-//% * If the DNS iterator ID doesn't exist.
+//% * If the TCP listener ID doesn't exist.
 fn drop_tcp_listener(mut caller: Caller<ProcessState>, tcp_listener_id: u64) -> Result<(), Trap> {
     caller
         .data_mut()
@@ -428,6 +432,53 @@ fn drop_tcp_listener(mut caller: Caller<ProcessState>, tcp_listener_id: u64) ->
     Ok(())
 }
 
+//% lunatic::networking::local_addr(tcp_listener_id: i64, id_u64_ptr: u32) -> i64
+//%
+//% Returns the local address that this listener is bound to as an DNS iterator with just one
+//% element.
+//% * 0 on success - The local address that this listener is bound to is returned as an DNS
+//%                  iterator with just one element and written to **id_ptr**.
+//%
+//% * 1 on error   - The error ID is written to **id_u64_ptr**
+//%
+//% Traps:
+//% * If the tcp listener ID doesn't exist.
+//% * If **peer_socket_addr_id_ptr** is outside the memory.
+fn local_addr(
+    mut caller: Caller<ProcessState>,
+    tcp_listener_id: u64,
+    id_u64_ptr: u32,
+) -> Result<u32, Trap> {
+    let tcp_listener = caller
+        .data()
+        .resources
+        .tcp_listeners
+        .get(tcp_listener_id)
+        .or_trap("lunatic::network::local_addr: listener ID doesn't exist")?;
+    let (dns_iter_or_error_id, result) = match tcp_listener.local_addr() {
+        Ok(socket_addr) => {
+            let dns_iter_id = caller
+                .data_mut()
+                .resources
+                .dns_iterators
+                .add(DnsIterator::new(vec![socket_addr].into_iter()));
+            (dns_iter_id, 0)
+        }
+        Err(error) => (caller.data_mut().errors.add(error.into()), 1),
+    };
+
+    let memory = get_memory(&mut caller)?;
+    memory
+        .write(
+            &mut caller,
+            id_u64_ptr as usize,
+            &dns_iter_or_error_id.to_le_bytes(),
+        )
+        .or_trap("lunatic::network::local_addr")?;
+
+    Ok(result)
+}
+
 //% lunatic::networking::tcp_accept(
 //%     listener_id: u64,
 //%     id_u64_ptr: u32,
@@ -447,7 +498,7 @@ fn drop_tcp_listener(mut caller: Caller<ProcessState>, tcp_listener_id: u64) ->
 fn tcp_accept(
     mut caller: Caller<ProcessState>,
     listener_id: u64,
-    id_ptr: u32,
+    id_u64_ptr: u32,
     socket_addr_id_ptr: u32,
 ) -> Box<dyn Future<Output = Result<u32, Trap>> + Send + '_> {
     Box::new(async move {
@@ -475,7 +526,7 @@ fn tcp_accept(
         memory
             .write(
                 &mut caller,
-                id_ptr as usize,
+                id_u64_ptr as usize,
                 &tcp_stream_or_error_id.to_le_bytes(),
             )
             .or_trap("lunatic::networking::tcp_accept")?;
@@ -550,12 +601,7 @@ fn tcp_connect(
             Ok(result)
         } else {
             // Call timed out
-            let error = std::io::Error::new(std::io::ErrorKind::TimedOut, "Connect timed out");
-            let error_id = caller.data_mut().errors.add(error.into());
-            memory
-                .write(&mut caller, id_u64_ptr as usize, &error_id.to_le_bytes())
-                .or_trap("lunatic::networking::tcp_connect")?;
-            Ok(1)
+            Ok(9027)
         }
     })
 }
@@ -671,12 +717,7 @@ fn tcp_write_vectored(
             Ok(return_)
         } else {
             // Call timed out
-            let error = std::io::Error::new(std::io::ErrorKind::TimedOut, "Write call timed out");
-            let error_id = caller.data_mut().errors.add(error.into());
-            memory
-                .write(&mut caller, opaque_ptr as usize, &error_id.to_le_bytes())
-                .or_trap("lunatic::networking::tcp_write_vectored")?;
-            Ok(1)
+            Ok(9027)
         }
     })
 }
@@ -692,6 +733,7 @@ fn tcp_write_vectored(
 //% Returns:
 //% * 0 on success - The number of bytes read is written to **opaque_ptr**
 //% * 1 on error   - The error ID is written to **opaque_ptr**
+//% * 9027 if the operation timed out
 //%
 //% Reads data from TCP stream and writes it to the buffer.
 //%
@@ -739,12 +781,7 @@ fn tcp_read(
             Ok(return_)
         } else {
             // Call timed out
-            let error = std::io::Error::new(std::io::ErrorKind::TimedOut, "Read call timed out");
-            let error_id = caller.data_mut().errors.add(error.into());
-            memory
-                .write(&mut caller, opaque_ptr as usize, &error_id.to_le_bytes())
-                .or_trap("lunatic::networking::tcp_read")?;
-            Ok(1)
+            Ok(9027)
         }
     })
 }
